@@ -343,17 +343,32 @@ def commit(
     return cast(dict, _post(base_url + url, auth, json=operations, ssl_verify=ssl_verify, params=params))
 
 
-def get_diff(base_url: str, auth: Optional[AuthBase], from_ref: str, to_ref: str, ssl_verify: bool = True) -> dict:
+def get_diff(
+    base_url: str,
+    auth: Optional[AuthBase],
+    from_ref: str,
+    to_ref: str,
+    from_hash_on_ref: Optional[str] = None,
+    to_hash_on_ref: Optional[str] = None,
+    ssl_verify: bool = True,
+) -> dict:
     """Fetch the diff for two given references.
 
     :param base_url: base Nessie url
     :param auth: Authentication settings
     :param from_ref: the starting ref for the diff
     :param to_ref:  the ending ref for the diff
+    :param from_hash_on_ref: optional hash on from reference
+    :param to_hash_on_ref: optional hash on to reference
     :param ssl_verify: ignore ssl errors if False
     :return: json dict of a Diff
     """
-    return cast(dict, _get(f"{base_url}/diffs/{from_ref}...{to_ref}", auth, ssl_verify=ssl_verify))
+    from_hash_on_ref_asterisk = f"*{from_hash_on_ref}" if from_hash_on_ref else ""
+    to_hash_on_ref_asterisk = f"*{to_hash_on_ref}" if to_hash_on_ref else ""
+    return cast(
+        dict,
+        _get(f"{base_url}/diffs/{from_ref}{from_hash_on_ref_asterisk}...{to_ref}{to_hash_on_ref_asterisk}", auth, ssl_verify=ssl_verify),
+    )
 
 
 def list_reflog(
